#ifndef _level_hh_
#define _level_hh_
#define SIZE 13*23
#define WORLDDIR "./worlds/"
#define LEVELDIR "./levels/"

typedef unsigned char uchar;

struct properties
{
  char n[30];	// Name
  char a[30];	// Author
  char c[30];	// Comment
  char b[255];	// Background
  int f[5]; 	// Field ( 0 = time, 1 = increase, 2 = start speed )
};

class World;

class Level
{
friend class World;
  
public:

  Level();			// Constructor
  Level(const Level& l);	// Copy-constructor
  ~Level();			// Destructor

  // Operators

  Level& operator = (const Level& l);

  // Level operations

  void Update(uchar *v);	// Update level with data from v
  void Insert(uchar *v);	// Fill v with data from level
  void SetProperties(properties p);
  void GetProperties(properties *p);

  // Debugging operations

  void Dump(int i);		// Dump all data to tty

private:
  void Load(int fd);	// Load level
  void Save(int fd);	// Save level
  uchar data[13*23];
  struct properties props;
  char del;
};


class World
{
public:
  World();		// Constructor
  ~World();		// Destructor

  // World operations  
  
  void Load(char *n);				// Load world
  void Save(char *n, uchar *order);		// Save world
  void SetProperties(struct properties p);	// Set properties
  void GetProperties(struct properties *p);	// Get properties

  // Level operations
  
  uchar InsertLevel(Level &l);
  void SaveLevel(int l);		// Save level l
  uchar LoadLevel(char *n);		// Load level l
  void DeleteLevel(int p);		// Delete level p in order
  void LevelCopy(int l, uchar *v);	// Copy data from v to level l
  void LevelCopy(uchar *v, int l);	// Copy data from level l to v
  void SetLevelProperties(int l, struct properties p);
  void GetLevelProperties(int l, struct properties *p);
  uchar Levels();			// return number of levels
  
  // Debugging operations
  
  void DumpAll(int k);		// Dump all levels
  void Dump(int n);		// Dump level n

private:
  void Add(Level &l);	// returns address
  Level *levels;
  uchar total;		// Number of levels in order
  struct properties props;
};


#endif


/*
   Leveldata: 13 x 23
*/













