#include "declarations.hh"
#include "gfm.hh"

// Menu_world NEW

Menu_item menu_world_new(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    if (edited)
    {
      int result = notice_prompt(Led_win_main.win_main, NULL,
				 NOTICE_MESSAGE_STRINGS, "World has been edited",
				 "Do you want to discard edits?", NULL,
				 NOTICE_BUTTON_YES, "Yes",
				 NOTICE_BUTTON_NO, "No",
				 NULL);
      if (result == NOTICE_NO)
	return item;
    }
    edited = 0;
    back = PIX_CLR;
    // Allocations

    filename[0] = '\0';
    cur = 0;
    delete world;
    ClearList();
    world = new World;

    // Initializations

    InsertLevel(top);
    SetLevelProperties();
    
    // Screen
    
    DrawLevel();
    DisableMenuItem(Led_win_main.btn_level, "Move");
    DisableMenuItem(Led_win_main.btn_world, "Save");
    EnableMenuItem(Led_win_main.btn_world, "Save as");
    EnableMenuItem(Led_win_main.btn_world, "Properties");
    strcpy(tmp, "New world '");
    strcat(tmp, props.n);
    strcat(tmp, "' created ok."); 
    Message(Led_win_main.win_main, tmp);
    world->GetProperties(&props);
    xv_set(Led_win_main.list_world, PANEL_LIST_TITLE, props.n, PANEL_INACTIVE, FALSE, NULL);
    xv_set(Led_win_main.btn_level, PANEL_INACTIVE, FALSE, NULL);
    SetLevels();
    break;
  }

  return item;
}


// Menu_world LOAD

Menu_item menu_world_load(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    if (edited)
    {
      int result = notice_prompt(Led_win_main.win_main, NULL,
				 NOTICE_MESSAGE_STRINGS, "World has been edited",
				 "Do you want to discard edits?", NULL,
				 NOTICE_BUTTON_YES, "Yes",
				 NOTICE_BUTTON_NO, "No",
				 NULL);
      if (result == NOTICE_NO)
	return item;
    }
    edited = 0;
    rload = gfm_initialize(NULL, NULL, "Load World");
    gfm_activate(rload, worlddir, "^.*\\.world$", NULL, cbload,NULL, GFM_LOAD);
    break;
  }
  return item;
}


// Menu_world SAVE

Menu_item menu_world_save(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    EnableMenuItem(Led_win_main.btn_world, "Save as");
    world->LevelCopy(order[SELECTED], brick);
    strcpy(props.n, (char*)xv_get(Led_win_world.txt_world_name, PANEL_VALUE));
    strcpy(props.a, (char*)xv_get(Led_win_world.txt_world_author, PANEL_VALUE));
    strcpy(props.c, (char*)xv_get(Led_win_world.txt_world_comment, PANEL_VALUE));
    world.SetProperties(props);
    world->Save(filename, order);
    strcpy(tmp, "World saved ok."); 
    Message(Led_win_main.win_main, tmp);
    edited = 0;
    break;
  }
  return item;
}


// Menu_world SAVE_AS

Menu_item menu_world_save_as(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    rsave = gfm_initialize(NULL, NULL, "Save World as");
    gfm_activate(rsave, worlddir, "^.*\\.world$", NULL, cbsave, NULL, GFM_SAVE);
    break;
  }
  return item;
}


// Menu_world QUIT

Menu_item menu_world_quit(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    if (edited)
    {
      int result = notice_prompt(Led_win_main.win_main, NULL,
				 NOTICE_MESSAGE_STRINGS, "World has been edited",
				 "Do you want to discard edits?", NULL,
				 NOTICE_BUTTON_YES, "Yes",
				 NOTICE_BUTTON_NO, "No",
				 NULL);
      if (result == NOTICE_NO)
	return item;
    }
    exit(0);
    break;
  }
  return item;
}


// Menu_world PROPERTIES

Menu_item menu_world_properties(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    SetWorldProperties();
    int x = xv_get(Led_win_main.win_main, XV_X);
    int y = xv_get(Led_win_main.win_main, XV_Y);
    xv_set(Led_win_world.win_world, XV_X, x+50, XV_Y, y+165, NULL);
    xv_set(Led_win_world.win_world, XV_SHOW, TRUE, NULL);
    DisableMenuItem(Led_win_main.btn_world, "Properties");
    break;
  }
  return item;
}

// Menu_level DELETE

Menu_item menu_level_delete(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    int k = SELECTED;

    strcpy(tmp, "Level '");
    strcat(tmp, (char*)xv_get(Led_win_main.list_world, PANEL_LIST_STRING, k));
    strcat(tmp, "' deleted ok."); 
    Message(Led_win_main.win_main, tmp);

    DeleteOrder(k);
    world->DeleteLevel(order[k]);
    SetLevels();

    xv_set(Led_win_main.list_world, PANEL_LIST_DELETE, SELECTED, NULL);

    if (NROWS == 0) // Om man har tagit bort alla
    {
      DisableMenuItem(Led_win_main.btn_level, "Properties");
      DisableMenuItem(Led_win_main.btn_level, "Move");
      DisableMenuItem(Led_win_main.btn_level, "Delete");
      if (xv_get(Led_win_edit.win_edit, XV_SHOW) == TRUE)
      {
	memset(brick, 0, GAME_HEIGHT*GAME_WIDTH);
	DrawLevel();
	props.n[0] = props.a[0] = props.c[0] ='\0';
	SetLevelProperties();
	xv_set(Led_win_edit.win_edit, XV_LABEL, (char*)'\0', NULL);
      }
    }
    else // Om det fortfaranade finns kvar i listan
    {
      if (k > 0) k--;
      SELECT(k);
      world->GetLevelProperties(order[k], &props);
      SetLevelProperties();
      world->LevelCopy(brick, order[k]);
      InitBackground(order[k]);
      DrawLevel();
    }
    edited = 1;
    break;
  }
  return item;
}


// Menu_level PROPERTIES

Menu_item menu_level_properties(Menu_item item, Menu_generate op)
{
  int x = xv_get(Led_win_main.win_main, XV_X);
  int y = xv_get(Led_win_main.win_main, XV_Y);
  switch (op) {
  case MENU_NOTIFY:
    xv_set(Led_win_level.win_level, XV_X, x+50, XV_Y, y+165, NULL);
    xv_set(Led_win_level.win_level, XV_SHOW, TRUE, NULL);
    DisableMenuItem(Led_win_main.btn_level, "Properties");
    break;
  }
  return item;
}

// Menu_level LOAD

Menu_item menu_level_load(Menu_item item, Menu_generate op)
{
  switch (op) 
  {
  case MENU_NOTIFY:
    rload = gfm_initialize(NULL, NULL, "Load Level");
    gfm_activate(rload, leveldir, NULL, NULL, cbloadl, NULL, GFM_LOAD);
    break;
  }
  return item;
}

// Menu_level SAVE

Menu_item menu_level_save(Menu_item item, Menu_generate op)
{
  switch (op) 
  {
  case MENU_NOTIFY:
    world->SaveLevel(order[SELECTED]);
    Message(Led_win_main.win_main, "Level saved ok.");
    break;
  }
  return item;
}


// Menu_insert BEFORE

Menu_item menu_insert_before(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    world->LevelCopy(order[SELECTED], brick); // save current bricks b4 switch
    InsertLevel(before);
    world->LevelCopy(brick, order[SELECTED]);
    DrawLevel();
    break;
  }
  return item;
}


// Menu_insert AFTER

Menu_item menu_insert_after(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    world->LevelCopy(order[SELECTED], brick); // save current bricks b4 switch
    InsertLevel(after);
    world->LevelCopy(brick, order[SELECTED]);
    DrawLevel();
    break;
  }
  return item;
}


// Menu_insert TOP

Menu_item menu_insert_top(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    world->LevelCopy(order[SELECTED], brick); // save current bricks b4 switch
    InsertLevel(top);
    world->LevelCopy(brick, order[SELECTED]);
    DrawLevel();
    break;
  }
  return item;
}


// Menu_insert BOTTOM

Menu_item menu_insert_bottom(Menu_item item, Menu_generate op)
{
  switch (op) {
  case MENU_NOTIFY:
    world->LevelCopy(order[SELECTED], brick); // save current bricks b4 switch
    InsertLevel(bottom);
    world->LevelCopy(brick, order[SELECTED]);
    DrawLevel();
    break;
  }
  return item;
}


// Menu_move UP

Menu_item menu_move_up(Menu_item item, Menu_generate op)
{
  if (SELECTED == 0) return item;
		   
  switch (op) {
  case MENU_NOTIFY:
    world->GetLevelProperties(order[SELECTED], &props);
    uchar k = order[SELECTED-1];
    order[SELECTED-1] = order[SELECTED];
    order[SELECTED] = k;
    xv_set(Led_win_main.list_world, PANEL_LIST_STRING, SELECTED, (char*)xv_get(Led_win_main.list_world, PANEL_LIST_STRING, SELECTED-1), NULL);
    xv_set(Led_win_main.list_world, PANEL_LIST_STRING, SELECTED-1, props.n, NULL);
    SELECT(SELECTED-1);
    world->GetLevelProperties(order[SELECTED], &props);
    SetLevelProperties();
    SetLevels();
    break;
  }
  return item;
}


// Menu_move DOWN

Menu_item menu_move_down(Menu_item item, Menu_generate op)
{
  if (SELECTED == NROWS-1) return item;
  
  switch (op) {
  case MENU_NOTIFY:
    world->GetLevelProperties(order[SELECTED], &props);
    uchar k = order[SELECTED+1];
    order[SELECTED+1] = order[SELECTED];
    order[SELECTED] = k;
    xv_set(Led_win_main.list_world, PANEL_LIST_STRING, SELECTED, (char*)xv_get(Led_win_main.list_world, PANEL_LIST_STRING, SELECTED+1), NULL);
    xv_set(Led_win_main.list_world, PANEL_LIST_STRING, SELECTED+1, props.n, NULL);
    SELECT(SELECTED+1);
    world->GetLevelProperties(order[SELECTED], &props);
    SetLevelProperties();
    SetLevels();
    break;
  }
  return item;
}

// Disable menu item

void DisableMenuItem(Panel_item item, char *n)
{
  Menu_item mi = (Menu_item) xv_find((Menu) xv_get(item, PANEL_ITEM_MENU), MENUITEM, MENU_STRING, n, NULL);
  xv_set(mi, MENU_INACTIVE, TRUE, NULL);
}


// Enable menu item
    
void EnableMenuItem(Panel_item item, char *n)
{
  Menu_item mi = (Menu_item) xv_find((Menu) xv_get(item, PANEL_ITEM_MENU), MENUITEM, MENU_STRING, n, NULL);
  xv_set(mi, MENU_INACTIVE, FALSE, NULL);
}

