#include "level.hh"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <memory.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


/*

   Implementation of class Level

*/

// Constructor

Level :: Level() 
{
  del = 0;
  strcpy(props.n, "Untitled");
  memset(data, 0, SIZE);
  props.f[0] = 300;
  props.f[1] = 50;
  props.f[2] = 4;
  for (int y = 3; y < 5; y++) props.f[y] = 0;
  for (y = 0; y < 30; y++) props.b[y] = props.c[y] = props.a[y] = '\0';
  for (y = 0; y < 13; y++) data[y] = 10;
  for (y = 0; y < 23; y++) data[y*13] = 10, data[y*13+12] = 10;
}


// Copy-constructor

Level :: Level(const Level& l)
{
  strcpy(props.n, l.props.n);
  strcpy(props.a, l.props.a);
  strcpy(props.c, l.props.c);
  strcpy(props.b, l.props.b);
  del = l.del;
  memcpy(data, l.data, SIZE);
  for (int y = 0; y < 5; y++) props.f[y] = l.props.f[y];
}


// Destructor

Level :: ~Level() 
{
}


// Operator =

Level &Level :: operator = (const Level& l)
{
  strcpy(this->props.n, l.props.n);
  strcpy(this->props.a, l.props.a);
  strcpy(this->props.c, l.props.c);
  strcpy(this->props.b, l.props.b);
  this->del = l.del;
  for (int y = 0; y < 5; y++) this->props.f[y] = l.props.f[y];
  memcpy(this->data, l.data, SIZE);
  return *this;
}


// Load level 

void Level :: Load(int fd)
{
  read(fd, (char*)&props, sizeof(props));
  read(fd, (char*)&data, sizeof(data));
}


// Save level

void Level :: Save(int fd)
{
  write(fd, (char*)&props, sizeof(props));
  write(fd, (char*)&data, sizeof(data));
}


// Update level with data from v

void Level :: Update(unsigned char *v)
{
  memcpy(data, v, SIZE);
}


// Update v with data from level

void Level :: Insert(uchar *v)
{
  memcpy(v, data, SIZE);
}


// Set properties

void Level :: SetProperties(struct properties p)
{
  strcpy(props.n, p.n);
  strcpy(props.a, p.a);
  strcpy(props.c, p.c);
  strcpy(props.b, p.b);
  for (int k = 0; k < 5; k++) props.f[k] = p.f[k];
}


// Get properties

void Level :: GetProperties(struct properties *p)
{
  strcpy(p->n, props.n);
  strcpy(p->a, props.a);
  strcpy(p->c, props.c);
  strcpy(p->b, props.b);
  for (int k = 0; k < 5; k++) p->f[k] = props.f[k];
}


// Dump level information to tty

void Level :: Dump(int i = 0)
{
  printf("Level: \t\t%s\n", props.n);
  printf("Author: \t%s\n", props.a);
  printf("Comment: \t%s\n", props.c);
  printf("Background: \t%s\n", props.b);
  printf("Deleted: \t%s\n", del==1?"Yes":"No");
  printf("Fields: \t");
  for (int y = 0; y < 5; y++)
    printf("%d ", props.f[y]);
  puts(" ");

  if (i == 0)
  {
    printf("\n");
    for (int y = 0; y < 23; y++, printf("\n"))
      for (int x = 0; x < 13; x++)
	printf("%02d ", data[y*13+x]);
  }
  printf("--------------------------------------\n");
}


/*

   Implementation of class World

*/


// Constructor

World :: World()
{
  total = 0;
  strcpy(props.n, "Untitled");
  strcpy(props.a, "");
  strcpy(props.c, "");
  strcpy(props.b, "");
}


// Destructor

World :: ~World()
{
  delete levels;
}


// Load world

void World :: Load(char *n)
{
  Level l;
  uchar t;
  char b[255];
  strcpy(b, WORLDDIR);
  strcat(b, n);

  int fd = open(b, O_RDONLY);
  delete levels;
  total = 0;

  read(fd, (char*)&props, sizeof(props));
  read(fd, (char*)&t, sizeof(total));

  for (int j = 0; j < t; j++)
  {
    l.Load(fd);
    Add(l);
  } 
  close(fd);
}


// Save world

void World :: Save(char *n, uchar *order)
{
  char t[255];
  strcpy(t, WORLDDIR);
  strcat(t, n);

  unlink(t);
  int fd = open(t, O_WRONLY | O_CREAT, S_IWUSR|S_IRUSR|S_IRGRP|S_IROTH);

  write(fd, (char*)&props, sizeof(props));
  write(fd, (char*)&total, sizeof(total));

  for (int k = 0; k < total; k++)
  {
    levels[order[k]].Save(fd);
    if (levels[order[k]].del)
      printf("Deleted\n");
  }

  close(fd);
}


// Set properties

void World :: SetProperties(struct properties p)
{
  strcpy(props.n, p.n);
  strcpy(props.a, p.a);
  strcpy(props.c, p.c);
}


// Get properties

void World :: GetProperties(struct properties *p)
{
  strcpy(p->n, props.n);
  strcpy(p->a, props.a);
  strcpy(p->c, props.c);
}


// Return number of levels

uchar World :: Levels()
{
  return total;
}


// Insert level l

uchar World :: InsertLevel(Level &l)
{
  Add(l); // allocate memory
  return total - 1;
}


// Delete level p

void World :: DeleteLevel(int p)
{
  if (p > total) return;
  levels[p].del = 1;
  total--;
}


// Copy data from v to level l

void World :: LevelCopy(int l, uchar *v)
{
  levels[l].Update(v);
}


// Copy data from level l to v

void World :: LevelCopy(uchar *v, int l)
{
  levels[l].Insert(v);
}


// Set properties for level l

void World :: SetLevelProperties(int l, struct properties p)
{
  levels[l].SetProperties(p);
}


// Get properties for level l

void World :: GetLevelProperties(int l, struct properties *p)
{
  levels[l].GetProperties(p);
}


void World :: Dump(int n)
{
  levels[n].Dump();
}

void World :: DumpAll(int k = 0)
{
  printf("\n--------------------------------------\n");
  printf("World: \t\t%s\n", props.n);
  printf("Author: \t%s\n", props.a);
  printf("Comment: \t%s\n", props.c);
  printf("Total: \t\t%d\n", total);
  printf("--------------------------------------\n");

  for (int i = 0; i < total; i++)
  {
    printf("Internal: \t%d\n", i);
    levels[i].Dump(k);
  }
}

void World :: Add(Level &l)
{
  Level *tmp = new Level[total+1];

  for (int k = 0; k < total; k++)
    tmp[k] = levels[k];

  tmp[k] = l;
  total++;
  
  delete levels;
  
  levels = new Level[total];

  for (k = 0; k < total; k++)
    levels[k] = tmp[k];

  delete tmp;
}

void World :: SaveLevel(int l)
{
  char t[255];
  strcpy(t, LEVELDIR);
  strcat(t, levels[l].props.n);
  unlink(t);
  int fd = open(t, O_WRONLY | O_CREAT, S_IWUSR|S_IRUSR|S_IRGRP|S_IROTH);
  levels[l].Save(fd);
  close(fd);
}

uchar World :: LoadLevel(char *n)
{
  char t[255];
  strcpy(t, LEVELDIR);
  strcat(t, n);
  Level l;
  int fd = open(t, O_RDONLY);
  l.Load(fd);
  Add(l);
  close(fd);
  return total-1;
}

